#include <stdio.h>
#include <windows.h>
#include "COM-DAFG.h"

int main()
	{
	// check the DLL version:
	if ((COM_DAFG_Version() & 0xFF00) != 0x0100)
		{
		printf ("Wrong DLL Version: expected 1.x, found %d.%02d\n",
			COM_DAFG_Version() / 0x100, COM_DAFG_Version() & 0xFF);
		return -1;
		}

	// open the port:
	if (COM_DAFG_Open())
		{
		// handle a possible error:
		printf ("%s\n", COM_DAFG_ErrorMessage());
		return 1;
		}

	// list waveforms and save them:
	for (WORD WaveNum = 0; WaveNum < WAVEFORM_NUMBER; WaveNum++)
		{
		// get the waveform parameters:
		WORD Origin, Size; double Periods;
		char Name [WAVEFORM_NAME_SIZE+1];
		if (COM_DAFG_GetWaveform (WaveNum, Origin, Size,
			Periods, Name))
			{
			// handle a possible error:
			printf ("%s\n", COM_DAFG_ErrorMessage());
			return 1;
			}
		// is the waveform empty?
		if (!Size)
			{
			if (WaveNum == 0)
				printf ("No waveforms found\n");
			break;
			}
		// print the waveform parameters:
		printf ("Waveform number %2hd: Size: %5d, "
			"Periods: %8.5lf, Name: %s\n", WaveNum, 1+Size,
			Periods, Name);
		// generate the file name:
		char FileName [0x10];
		sprintf (FileName, "%02hd.txt", WaveNum);
		// open the file:
		FILE * const DataFile = fopen (FileName, "wt");
		if (DataFile == NULL)
			{
			// handle a possible error:
			printf ("Error opening the data file %s\n", FileName);
			return -1;
			}
		printf ("Saving waveform into data file %s\n",
			WaveNum, FileName);
		// loop for all data segments:
		for (WORD Base = 0;
			Base <= Size/MAX_WAVEFORM_BLOCK_SIZE;
			Base++)
			{
			WORD DataSize; WORD Data [MAX_WAVEFORM_BLOCK_SIZE];
			// get a data segment:
			if (COM_DAFG_GetWaveformData (WaveNum, Base,
				DataSize, Data))
				// on error, retry the operation:
				if (COM_DAFG_GetWaveformData (WaveNum, Base,
					DataSize, Data))
					{
					// handle a possible error:
					printf ("%s\n", COM_DAFG_ErrorMessage());
					fclose (DataFile);
					return -1;
					}
			// loop for all data in the segment:
			for (int n = 0; n < DataSize; n++)
				// output the data:
				if (fprintf (DataFile, "%.6lf\n",
					double (Data[n]-WAVEFORM_DATA_ZERO) /
						WAVEFORM_DATA_SCALE) == EOF)
					{
					// handle a possible error:
					printf ("Error writing the data file %s\n",
						FileName);
					return -1;
					}
			}
		// close the file:
		if (fclose (DataFile) == EOF)
			{
			printf ("Error closing the data file %s\n",
				FileName);
			}
		}

	//finish the program:
	return 0;
	}
